import { RouteRecordRaw } from 'vue-router';
import pinia from '@/stores/index';
import { useUserInfo } from '@/stores/userInfo';
import { useAppConfig } from '@/stores/appConfig';
import { useRequestOldRoutes } from '@/stores/requestOldRoutes';
import { Local, Cookie } from '@/utils/storage';
import { NextLoading } from '@/utils/loading';
import { dynamicRoutes, notFoundAndNoPower } from '@/router/route';
import { formatFlatteningRoutes, formatTwoStageRoutes, router } from '@/router/index';
import { useRoutesList } from '@/stores/routesList';
import { useTagsViewRoutes } from '@/stores/tagsViewRoutes';
import { getMenu } from '@/api/login/user';
import { removeNullAndDuplicates } from '@/utils/arrayOperation';
import other from '@/utils/other';
import { pwdCheck } from '@/api/login';
import { appConfigInfo } from '@/api/public/system';
import signalR from '@/utils/signalR';
const layoutModules: any = import.meta.glob('../layout/routerView/*.{vue,tsx}');
const viewsModules: any = import.meta.glob('../views/**/*.{vue,tsx}');
// 后端控制路由

/**
 * 获取目录下的 .vue、.tsx 全部文件
 * @method import.meta
 * @link 参考：https://cn.vitejs.dev/guide/features.html#json
 */
const dynamicViewsModules: Record<string, Function> = Object.assign({}, { ...layoutModules }, { ...viewsModules });

// 菜单格式化
const formatRouter = (arr: Array<any>): Array<any> => {
	if (!arr) return [];
	return arr.map((v: any) => {
		const isDynamic = v.path.indexOf('/:') > -1;
		v.name = v.ruleName;
		v.meta = {
			title: v.pageName,
			icon: v.icon,
			isHide: v.isHide,
			isKeepAlive: v.isKeepAlive,
			isAffix: v.isAffix,
			isLink: v.link,
			isIframe: v.isIframe,
			...(isDynamic && { isDynamic, isDynamicPath: v.path }),
		};
		if (v.children?.length) {
			v.children = formatRouter(v.children);
		}
		return v;
	});
};
// 获取系统配置 存入pinia
const getAppConfigFn = async () => {
	try {
		const { result } = await appConfigInfo();
		useAppConfig().setAppConfigInfo({
			isRestApproval: result.isRestApproval ?? false, //是否开启小休审批
			isNeedTelNo: result.isNeedTelNo ?? false, //分机签入是否需要选择号码
			talkingDealTime: result.talkingDealTime ?? 0, // 自动话后整理时间
			isTelNeedVerify: result.isTelNeedVerify ?? false, //分机签入是否需要填写密码
			isCustomEvent: result.isCustomEvent ?? false, //是否开启自定义事件
			isTranspondCity: result.isTranspondCity ?? false, //是否开启市州互转
			isAverageSendOrder: result.isAverageSendOrder ?? false, //是否开启平均派单
			noSignOrgCode: result.noSignOrgCode ?? [], //不参与会签的机构编码
			isOpenRepeatedWorkOrders: result.isOpenRepeatedWorkOrders ?? false, //是否开启重复工单
		});
		console.log(
			`是否开启小休审批${result.isRestApproval},自动话后整理时间${result.talkingDealTime}秒,
			分机签入是否需要选择号码${result.isNeedTelNo},分机签入是否需要填写密码${result.isTelNeedVerify},
			是否开启自定义事件${result.isCustomEvent},是否开启市州互转${result.isTranspondCity},是否开启平均派单${result.isAverageSendOrder},,不参与会签的机构编码${result.noSignOrgCode},是否开启重复工单${result.isOpenRepeatedWorkOrders}`
		);
	} catch (e) {
		console.log(e);
	}
};
// 检查是否修改过密码 如果没有修改 跳转到修改密码页面
const checkPwd = (): void => {
	pwdCheck().then((res: any) => {
		if (!res.result) {
			router.replace('/resetPwd');
		}
	});
};

/**
 * 后端控制路由：初始化方法，防止刷新时路由丢失
 * @method NextLoading 界面 loading 动画开始执行
 * @method useUserInfo().setUserInfos() 触发初始化用户信息 pinia
 * @method useRequestOldRoutes().setRequestOldRoutes() 存储接口原始路由（未处理component），根据需求选择使用
 * @method setAddRoute 添加动态路由
 * @method setFilterMenuAndCacheTagsViewRoutes 设置路由到 vuex routesList 中（已处理成多级嵌套路由）及缓存多级嵌套数组处理后的一维数组
 */

export async function initBackEndControlRoutes() {
	// 界面 loading 动画开始执行
	if (window.nextLoading === undefined) NextLoading.start();
	// 无 token 停止执行下一步
	if (!Cookie.get('token')) return false;

	let resRouter = <EmptyArrayType>[];
	if (Local.get('requestOldRoutes')) {
		//获取到缓存
		resRouter = Local.get('requestOldRoutes');
		// 触发初始化用户信息 pinia
		const authBtnList = Local.get('authBtn') ?? [];
		await useUserInfo().setUserInfos(authBtnList);
		// 无登录权限时，添加判断
		// https://gitee.com/lyt-top/vue-next-admin/issues/I64HVO
		if (resRouter?.length <= 0) return Promise.resolve(true);
	} else {
		// 获取路由菜单数据
		const { result } = await getBackEndControlRoutes();
		const buttons = result?.buttons ?? []; // 权限按钮列表
		// 无登录权限时，添加判断
		// https://gitee.com/lyt-top/vue-next-admin/issues/I64HVO
		let routerList = result?.menus ?? [];
		if (routerList <= 0) {
			Cookie.remove('token');
			return Promise.resolve(true);
		}
		// 路由内容格式化
		resRouter = formatRouter(routerList);
		// 存储接口原始路由（未处理component），根据需求选择使用
		await useRequestOldRoutes().setRequestOldRoutes(other.deepClone(resRouter));
		// 存入缓存
		Local.set('requestOldRoutes', resRouter);
		// 获取系统配置
		await getAppConfigFn();
		// 触发初始化用户信息 pinia
		await useUserInfo().setUserInfos(removeNullAndDuplicates(buttons));
		// 存入缓存
		Local.set('authBtn', removeNullAndDuplicates(buttons));
	}
	// 检查是否修改过密码
	checkPwd();
	//  signalR 初始化signalr
	signalR.init();
	// 处理路由（component），替换 dynamicRoutes（@/router/route）第一个顶级 children 的路由
	dynamicRoutes[0].children = await backEndComponent(resRouter); // 首页需要权限控制
	// dynamicRoutes[0].children = dynamicRoutes[0].children?.concat(await backEndComponent(resRouter)); //首页如果不需要权限控制
	// 添加动态路由
	await setAddRoute();
	// 设置路由到 vuex routesList 中（已处理成多级嵌套路由）及缓存多级嵌套数组处理后的一维数组
	setFilterMenuAndCacheTagsViewRoutes();
}

/**
 * 设置路由到 vuex routesList 中（已处理成多级嵌套路由）及缓存多级嵌套数组处理后的一维数组
 * @description 用于左侧菜单、横向菜单的显示
 * @description 用于 tagsView、菜单搜索中：未过滤隐藏的(isHide)
 */
export function setFilterMenuAndCacheTagsViewRoutes() {
	const storesRoutesList = useRoutesList(pinia);
	storesRoutesList.setRoutesList(dynamicRoutes[0].children as any);
	setCacheTagsViewRoutes();
}

/**
 * 缓存多级嵌套数组处理后的一维数组
 * @description 用于 tagsView、菜单搜索中：未过滤隐藏的(isHide)
 */
export function setCacheTagsViewRoutes() {
	const storesTagsView = useTagsViewRoutes(pinia);
	storesTagsView.setTagsViewRoutes(formatTwoStageRoutes(formatFlatteningRoutes(dynamicRoutes))[0].children);
}

/**
 * 处理路由格式及添加捕获所有路由或 404 Not found 路由
 * @description 替换 dynamicRoutes（@/router/route）第一个顶级 children 的路由
 * @returns 返回替换后的路由数组
 */
export function setFilterRouteEnd() {
	let filterRouteEnd: any = formatTwoStageRoutes(formatFlatteningRoutes(dynamicRoutes));
	// notFoundAndNoPower 防止 404、401 不在 layout 布局中，不设置的话，404、401 界面将全屏显示
	// 关联问题 No match found for location with path 'xxx'
	filterRouteEnd[0].children = [...filterRouteEnd[0].children, ...notFoundAndNoPower];
	return filterRouteEnd;
}

/**
 * 添加动态路由
 * @method router.addRoute
 * @description 此处循环为 dynamicRoutes（@/router/route）第一个顶级 children 的路由一维数组，非多级嵌套
 * @link 参考：https://next.router.vuejs.org/zh/api/#addroute
 */
export async function setAddRoute() {
	await setFilterRouteEnd().forEach((route: RouteRecordRaw) => {
		router.addRoute(route);
	});
}

/**
 * 请求后端路由菜单接口
 * @description isRequestRoutes 为 true，则开启后端控制路由
 * @returns 返回后端路由菜单数据
 */
export function getBackEndControlRoutes() {
	// 管理员 admin
	return getMenu();
}

/**
 * 重新请求后端路由菜单接口
 * @description 用于菜单管理界面刷新菜单（未进行测试）
 * @description 路径：/src/views/system/menu/component/Menu-add.vue
 */
export function setBackEndControlRefreshRoutes() {
	Local.remove('requestOldRoutes');
	getBackEndControlRoutes();
	window.location.reload();
}

/**
 * 后端路由 component 转换
 * @param routes 后端返回的路由表数组
 * @returns 返回处理成函数后的 component
 */
export function backEndComponent(routes: any) {
	if (!routes) return;
	return routes.map((item: any) => {
		if (item.component) item.component = dynamicImport(dynamicViewsModules, item.component as string);
		item.children && backEndComponent(item.children);
		return item;
	});
}

/**
 * 后端路由 component 转换函数
 * @param dynamicViewsModules 获取目录下的 .vue、.tsx 全部文件
 * @param component 当前要处理项 component
 * @returns 返回处理成函数后的 component
 */
export function dynamicImport(dynamicViewsModules: Record<string, Function>, component: string) {
	const keys = Object.keys(dynamicViewsModules);
	const matchKeys = keys.filter((key) => {
		const k = key.replace(/..\/views|../, '');
		return k.startsWith(`${component}`) || k.startsWith(`/${component}`);
	});
	if (matchKeys?.length === 1) {
		const matchKey = matchKeys[0];
		return dynamicViewsModules[matchKey];
	}
	if (matchKeys?.length > 1) {
		return false;
	}
}
